@@ -2,4 +2,6 @@ from .norm import Norm  # the order is sensitive because of `price_barrier` meth
 from .bsm import Bsm, BsmDisp
 from .cev import Cev
 from .sabr import SabrHagan2002, SabrLorig2017, SabrChoiWu2021H, SabrChoiWu2021P
+from .sabr_int import SabrUncorrChoiWu2021
 from .nsvh import Nsvh1
+from .multiasset import BsmSpreadKirk, BsmSpreadBjerksund2014, NormBasket, NormSpread, BsmBasketLevy1992, BsmMax2
